@@ -24,20 +24,22 @@
 	This file implements many of the main interface routines of the
 	Audio File Library.
 */
+
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <assert.h>
 
 #include "audiofile.h"
 #include "util.h"
 #include "afinternal.h"
+#include "afinternal.h"
+#include "block.h"
 
 int _af_parseaiff (AFfilehandle filehandle);
 int _af_parsewave (AFfilehandle filehandle);
@@ -49,8 +51,6 @@ int auSyncFile (AFfilehandle);
 
 void aiffOpenFileWrite (AFfilehandle);
 
-int _af_blockReadFrames (const AFfilehandle file, int track, void *samples, const int count);
-
 int afSetVirtualByteOrder (AFfilehandle file, int track, int byteorder)
 {
 	assert(file);
@@ -58,9 +58,31 @@ int afSetVirtualByteOrder (AFfilehandle file, int track, int byteorder)
 	assert(byteorder == AF_BYTEORDER_BIGENDIAN ||
 		byteorder == AF_BYTEORDER_LITTLEENDIAN);
 
-	file->virtualByteOrder = byteorder;
+	if (file == NULL)
+	{
+		_af_error(AF_BAD_FILEHANDLE);
+		return -1;
+	}
 
-	return 1; /* FIXME: return value never used in app code */
+	if (track != AF_DEFAULT_TRACK)
+	{
+		_af_error(AF_BAD_TRACKID);
+		return -1;
+	}
+
+	if (byteorder != AF_BYTEORDER_BIGENDIAN &&
+		byteorder != AF_BYTEORDER_LITTLEENDIAN)
+	{
+		_af_error(AF_BAD_BYTEORDER);
+		return -1;
+	}
+
+	/* Assert that the current virtual byte order is valid. */
+	assert(file->virtualByteOrder == AF_BYTEORDER_BIGENDIAN ||
+		file->virtualByteOrder == AF_BYTEORDER_LITTLEENDIAN);
+
+	file->virtualByteOrder = byteorder;
+	return 0;
 }
 
 int afGetByteOrder (AFfilehandle file, int track)
@@ -117,21 +139,36 @@ int afReadFrames(const AFfilehandle file, int track, void *samples, const int co
 	assert(samples);
 	assert(count >= 0);
 
+	if (file->compression != NULL)
+	{
+		AFframecount (*readFrames) (const AFfilehandle, int, void *, const int);
+
+		assert(file->compression->codec);
+
+		readFrames = file->compression->codec->readFrames;
+		assert(readFrames);
+
+		return (readFrames(file, track, samples, count));
+	}
+
 	return _af_blockReadFrames(file, track, samples, count);
 }
 
 /*
 	afGetFrameCount returns the total number of sample frames for a given
 	track in an audio file.
- */
-AFframecount afGetFrameCount(AFfilehandle file, int track)
+*/
+AFframecount afGetFrameCount (AFfilehandle file, int track)
 {
 	if (file != NULL)
 	{
 		return file->frameCount;
 	}
-
-	return 0;
+	else
+	{
+		_af_error(AF_BAD_FILEHANDLE);
+		return -1;
+	}
 }
 
 /* afSyncFile returns 0 on success and -1 on failure */
@@ -139,9 +176,20 @@ int afSyncFile (AFfilehandle file)
 {
 	assert(file != NULL);
 
-	/* It is an error to sync a null file. */
+	/* Assert that the file type is one which is handled. */
+	assert(file->fileFormat == AF_FILE_AIFFC ||
+		file->fileFormat == AF_FILE_AIFF ||
+		file->fileFormat == AF_FILE_WAVE ||
+		file->fileFormat == AF_FILE_NEXTSND);
+
+	/*
+		It is an error to sync a null file.
+	*/
 	if (file == NULL)
+	{
+		_af_error(AF_BAD_FILEHANDLE);
 		return -1;
+	}
 
 	if (file->fileFormat == AF_FILE_AIFFC)
 		return aiffSyncFile(file);
@@ -151,11 +199,18 @@ int afSyncFile (AFfilehandle file)
 		return waveSyncFile(file);
 	else if (file->fileFormat == AF_FILE_NEXTSND)
 		return auSyncFile(file);
-
-	return -1;
+	/*
+		If none of the above branches are taken, the filehandle must be
+		invalid.
+	*/
+	else
+	{
+		_af_error(AF_BAD_FILEHANDLE);
+		return -1;
+	}
 }
 
-/* afCloseFile returns 0 on success and -1 on failure */
+/* afCloseFile returns 0 on success and -1 on failure. */
 int afCloseFile (AFfilehandle file)
 {
 	int	result;
@@ -174,9 +229,9 @@ int afCloseFile (AFfilehandle file)
 	if (result != 0)
 		return result;
 
-	result = fclose(file->fp);
+	result = af_fclose(file->fh);
 
-	/* close returns -1 on error */
+	/* fclose returns -1 on error. */
 	if (result != 0)
 		return result;
 
@@ -199,6 +254,7 @@ int afCloseFile (AFfilehandle file)
 	if (file->instruments != NULL)
 		free(file->instruments);
 
+	/* Free memory associated with the miscellaneous chunk. */
 	if (file->miscellaneous != NULL)
 	{
 		free(file->miscellaneous);
@@ -209,15 +265,10 @@ int afCloseFile (AFfilehandle file)
 	return 0;
 }
 
-static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode, AFfilesetup setup)
+static AFfilehandle _afOpenFileWrite2 (AF_VirtualFile *vf, AFfilesetup setup)
 {
-	FILE			*fp;
 	AFfilehandle	filehandle;
 
-	fp = fopen(filename, mode);
-	if (fp == NULL)
-		return AF_NULL_FILEHANDLE;
-
 	filehandle = malloc(sizeof (struct _AFfilehandle));
 	if (filehandle == NULL)
 		return NULL;
@@ -227,7 +278,6 @@ static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode, AF
 	filehandle->sampleWidth = setup->sampleWidth;
 	filehandle->sampleFormat = setup->sampleFormat;
 	filehandle->byteOrder = setup->byteOrder;
-	filehandle->virtualByteOrder = setup->byteOrder;
 	filehandle->sampleRate = setup->sampleRate;
 	filehandle->dataStart = 0;
 	filehandle->frameCount = 0;
@@ -235,7 +285,7 @@ static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode, AF
 
 	filehandle->aesDataPresent = setup->aesDataPresent;
 
-	filehandle->fp = fp;
+	filehandle->fh = vf;
 	filehandle->markerCount = 0;
 	filehandle->markers = NULL;
 	filehandle->loopCount = 0;
@@ -243,6 +293,14 @@ static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode, AF
 	filehandle->instrumentCount = 0;
 	filehandle->instruments = NULL;
 
+#if WORDS_BIGENDIAN
+	filehandle->virtualByteOrder = AF_BYTEORDER_BIGENDIAN;
+#else
+	filehandle->virtualByteOrder = AF_BYTEORDER_LITTLEENDIAN;
+#endif
+
+	filehandle->compression = setup->compression;
+
 	if (setup->miscellaneousCount == 0)
 	{
 		filehandle->miscellaneousCount = 0;
@@ -262,6 +320,9 @@ static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode, AF
 	{
 		int	i;
 
+		filehandle->byteOrder = AF_BYTEORDER_BIGENDIAN;
+		filehandle->sampleFormat = AF_SAMPFMT_TWOSCOMP;
+
 		filehandle->markerCount = 4;
 		filehandle->markers = malloc(4 * sizeof (struct _Marker));
 		memcpy(filehandle->markers, setup->markers,
@@ -287,6 +348,14 @@ static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode, AF
 
 		aiffOpenFileWrite(filehandle);
 	}
+	else if (filehandle->fileFormat == AF_FILE_WAVE)
+	{
+		filehandle->byteOrder = AF_BYTEORDER_LITTLEENDIAN;
+	}
+	else if (filehandle->fileFormat == AF_FILE_NEXTSND)
+	{
+		filehandle->byteOrder = AF_BYTEORDER_BIGENDIAN;
+	}
 
 #ifdef DEBUG
 	_af_printfilehandle(filehandle);
@@ -295,21 +364,29 @@ static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode, AF
 	return filehandle;
 }
 
-AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setup)
+#if 0
+/* Not used any more */
+static AFfilehandle _afOpenFileWrite (const char *filename, const char *mode,
+	AFfilesetup setup)
 {
 	FILE			*fp;
-	AFfilehandle	filehandle;
-	char			data[12];
-
-	if (!strcmp(mode, "w"))
-		return _afOpenFileWrite(filename, mode, setup);
 
 	fp = fopen(filename, mode);
 	if (fp == NULL)
-	{
-		_af_error(AF_BAD_OPEN);
 		return AF_NULL_FILEHANDLE;
-	}
+
+	return _afOpenFileWrite2(af_virtual_file_new_for_file(fp), setup);
+}
+#endif
+
+AFfilehandle
+afOpenVirtualFile(AF_VirtualFile *vfile, const char *mode, AFfilesetup setup)
+{
+	AFfilehandle	filehandle;
+	char			data[12];
+
+	if(!strcmp(mode, "w"))
+	  return _afOpenFileWrite2 (vfile, setup);
 
 	filehandle = malloc(sizeof (struct _AFfilehandle));
 	if (filehandle == NULL)
@@ -318,10 +395,10 @@ AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setu
 		return NULL;
 	}
 
-	fread(data, 1, 12, fp);
-	fseek(fp, 0, SEEK_SET);
+	af_fread(data, 1, 12, vfile);
+	af_fseek(vfile, 0, SEEK_SET);
 
-	filehandle->fp = fp;
+	filehandle->fh = vfile;
 	filehandle->currentFrame = 0;
 	filehandle->markerCount = 0;
 	filehandle->markers = NULL;
@@ -332,6 +409,13 @@ AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setu
 	filehandle->miscellaneousCount = 0;
 	filehandle->miscellaneous = NULL;
 	filehandle->aesDataPresent = 0;
+	filehandle->compression = NULL;
+
+#if WORDS_BIGENDIAN
+	filehandle->virtualByteOrder = AF_BYTEORDER_BIGENDIAN;
+#else
+	filehandle->virtualByteOrder = AF_BYTEORDER_LITTLEENDIAN;
+#endif
 
 	if (!memcmp(data, "FORM", 4) && !memcmp(data + 8, "AIFC", 4))
 	{
@@ -342,7 +426,6 @@ AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setu
 		filehandle->fileFormat = AF_FILE_AIFFC;
 		filehandle->sampleFormat = AF_SAMPFMT_TWOSCOMP;
 		filehandle->byteOrder = AF_BYTEORDER_BIGENDIAN;
-		filehandle->virtualByteOrder = AF_BYTEORDER_BIGENDIAN;
 
 		_af_parseaiff(filehandle);
 	}
@@ -355,7 +438,6 @@ AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setu
 		filehandle->fileFormat = AF_FILE_AIFF;
 		filehandle->sampleFormat = AF_SAMPFMT_TWOSCOMP;
 		filehandle->byteOrder = AF_BYTEORDER_BIGENDIAN;
-		filehandle->virtualByteOrder = AF_BYTEORDER_BIGENDIAN;
 
 		_af_parseaiff(filehandle);
 	}
@@ -368,7 +450,6 @@ AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setu
 		filehandle->fileFormat = AF_FILE_WAVE;
 		filehandle->sampleFormat = AF_SAMPFMT_TWOSCOMP;
 		filehandle->byteOrder = AF_BYTEORDER_LITTLEENDIAN;
-		filehandle->virtualByteOrder = AF_BYTEORDER_LITTLEENDIAN;
 
 		_af_parsewave(filehandle);
 	}
@@ -380,7 +461,6 @@ AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setu
 
 		filehandle->fileFormat = AF_FILE_NEXTSND;
 		filehandle->byteOrder = AF_BYTEORDER_BIGENDIAN;
-		filehandle->virtualByteOrder = AF_BYTEORDER_BIGENDIAN;
 
 		_af_parseau(filehandle);
 	}
@@ -394,6 +474,20 @@ AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setu
 	return filehandle;
 }
 
+AFfilehandle afOpenFile(const char *filename, const char *mode, AFfilesetup setup)
+{
+	FILE			*fp;
+
+	fp = fopen(filename, mode);
+	if (fp == NULL)
+	{
+		_af_error(AF_BAD_OPEN);
+		return AF_NULL_FILEHANDLE;
+	}
+
+	return afOpenVirtualFile(af_virtual_file_new_for_file(fp), mode, setup);
+}
+
 int afGetFileFormat (AFfilehandle file, int *version)
 {
 	assert(file);
@@ -439,7 +533,6 @@ AFfileoffset afGetTrackBytes (AFfilehandle file, int track)
 */
 float afGetFrameSize (AFfilehandle file, int track, int stretch3to4)
 {
-        float frameSize;
 	assert(file);
 	assert(track == AF_DEFAULT_TRACK);
 
@@ -467,14 +560,23 @@ AFframecount afSeekFrame (AFfilehandle file, int track, AFframecount frame)
 	assert(file);
 	assert(track == AF_DEFAULT_TRACK);
 
+	if (frame == -1)
+	{
+		return afTellFrame(file, track);
+	}
+
 	/* frameSize represents the size of a frame in bytes. */
-	frameSize = file->channelCount * (file->sampleWidth / 8);
+	frameSize = file->channelCount * ((file->sampleWidth + 7) / 8);
 
 	file->currentFrame = frame;
 
-	fseek(file->fp, file->dataStart + frameSize * frame, SEEK_SET);
+	if (af_fseek(file->fh, file->dataStart + frameSize * frame, SEEK_SET) != 0)
+	{
+		_af_error(AF_BAD_LSEEK);
+		return -1;
+	}
 
-	return frame; /* FIXME: what is proper value here? */
+	return frame;
 }
 
 AFfileoffset afTellFrame (AFfilehandle file, int track)
