@@ -1,6 +1,6 @@
 /*
 	Audio File Library
-	Copyright (C) 1998, Michael Pruett <michael@68k.org>
+	Copyright (C) 1998-1999, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -25,14 +25,14 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <sys/types.h>
 #include <stdio.h>
 #include <assert.h>
+
 #include "afinternal.h"
-#include "ulaw.h"
 #include "byteorder.h"
 
 /*
@@ -54,23 +54,31 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 	assert(samples);
 	assert(frameCount >= 0);
 
+#ifdef DEBUG
+	printf("**currentFrame = %ld\n", file->currentFrame);
+	printf("**file->frameCount = %ld\n", file->frameCount);
+	printf("**requested frameCount = %d\n", frameCount);
+#endif
+
 	frameSize = (file->sampleWidth + 7) / 8 * file->channelCount;
 
 	channelCount = file->channelCount;
 
-	sampleCount = frameCount * file->channelCount;
+	af_fseek(file->fh, file->dataStart + file->currentFrame * frameSize, SEEK_SET);
 
-	/* u-law takes 1-byte input, but gives 2-byte output, adjust offset for it */
-	if ( file->sampleFormat == AF_SAMPFMT_ISDNULAW )
-	    fseek(file->fp, 
-		  file->dataStart + file->currentFrame * frameSize/2, 
-		  SEEK_SET);
-	else
-	    fseek(file->fp, 
-		  file->dataStart + file->currentFrame * frameSize, 
-		  SEEK_SET);
+	assert(file->currentFrame <= file->frameCount);
 
-	file->currentFrame += frameCount;
+	if (file->currentFrame + frameCount > file->frameCount)
+		frameCount = file->frameCount - file->currentFrame;
+
+	assert(file->currentFrame + frameCount <= file->frameCount);
+
+	sampleCount = frameCount * channelCount;
+
+#ifdef DEBUG
+	printf("**adjusted frameCount = %d\n", frameCount);
+	printf("**adjusted sampleCount = %d\n", sampleCount);
+#endif
 
 	if (file->sampleWidth <= 8)
 	{
@@ -81,8 +89,11 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 			case AF_SAMPFMT_TWOSCOMP:
 				while (done < sampleCount)
 				{
-					if (fread(&datum, 1, 1, file->fp) < 1)
+					if (af_fread(&datum, 1, 1, file->fh) < 1)
+					{
+						_af_error(AF_BAD_READ);
 						break;
+					}
 					*buffer8++ = datum;
 					done++;
 				}
@@ -91,8 +102,11 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 			case AF_SAMPFMT_UNSIGNED:
 				while (done < sampleCount)
 				{
-					if (fread(&datum, 1, 1, file->fp) < 1)
+					if (af_fread(&datum, 1, 1, file->fh) < 1)
+					{
+						_af_error(AF_BAD_READ);
 						break;
+					}
 					/* Convert datum to a signed integer. */
 					datum ^= 128;
 					*buffer8++ = datum;
@@ -114,8 +128,11 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 			case AF_SAMPFMT_TWOSCOMP:
 				while (done < sampleCount)
 				{
-					if (fread(&datum, 2, 1, file->fp) < 1)
+					if (af_fread(&datum, 2, 1, file->fh) < 1)
+					{
+						_af_error(AF_BAD_READ);
 						break;
+					}
 					if (file->byteOrder != file->virtualByteOrder)
 						datum = _af_byteswapint16(datum);
 					*buffer16++ = datum;
@@ -126,8 +143,11 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 			case AF_SAMPFMT_UNSIGNED:
 				while (done < sampleCount)
 				{
-					if (fread(&datum, 2, 1, file->fp) < 1)
+					if (af_fread(&datum, 2, 1, file->fh) < 1)
+					{
+						_af_error(AF_BAD_READ);
 						break;
+					}
 					if (file->byteOrder != file->virtualByteOrder)
 						datum = _af_byteswapint16(datum);
 					/* Convert datum to a signed integer. */
@@ -137,16 +157,6 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 				}
 				break;
 
-			case AF_SAMPFMT_ISDNULAW:
-				while (done < sampleCount)
-				{
-					if (fread(&datum, 1, 1, file->fp) < 1)
-						break;
-					*buffer16++ = _af_ulaw2linear(datum);
-					done++;
-				}
-				break;
-
 			default:
 				_af_error(AF_BAD_SAMPFMT);
 				break;
@@ -162,8 +172,11 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 			case AF_SAMPFMT_TWOSCOMP:
 				while (done < sampleCount)
 				{
-					if (fread(threeBytes, 3, 1, file->fp) < 1)
+					if (af_fread(threeBytes, 3, 1, file->fh) < 1)
+					{
+						_af_error(AF_BAD_READ);
 						break;
+					}
 
 					datum = threeBytes[2] |
 						(threeBytes[1] << 8) |
@@ -200,8 +213,12 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 			case AF_SAMPFMT_TWOSCOMP:
 				while (done < sampleCount)
 				{
-					if (fread(&datum, 4, 1, file->fp) < 1)
+					if (af_fread(&datum, 4, 1, file->fh) < 1)
+					{
+						_af_error(AF_BAD_READ);
 						break;
+					}
+
 					if (file->byteOrder != file->virtualByteOrder)
 						datum = _af_byteswapint32(datum);
 					*buffer32++ = datum;
@@ -212,8 +229,12 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 			case AF_SAMPFMT_UNSIGNED:
 				while (done < sampleCount)
 				{
-					if (fread(&datum, 4, 1, file->fp) < 1)
+					if (af_fread(&datum, 4, 1, file->fh) < 1)
+					{
+						_af_error(AF_BAD_READ);
 						break;
+					}
+
 					if (file->byteOrder != file->virtualByteOrder)
 						datum = _af_byteswapint32(datum);
 					/* Convert datum to a signed integer. */
@@ -229,6 +250,20 @@ int _af_blockReadFrames (AFfilehandle file, int track, void *samples,
 		}
 	}
 
+#ifdef DEBUG
+	printf("**sampleCount = %d\n", sampleCount);
+	printf("**done = %d\n", done);
+	printf("**done/channelcount = %d\n", done/channelCount);
+#endif
+
+	/*
+		Make done represent the number of sample frames rather than samples.
+		Update the file's current frame to reflect how many sample
+		frames were actually read.
+	*/
+
 	done /= channelCount;
+	file->currentFrame += done;
+
 	return done;
 }
