@@ -1,6 +1,6 @@
 /*
 	Audio File Library
-	Copyright (C) 1998, Michael Pruett <michael@68k.org>
+	Copyright (C) 1998-1999, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -28,15 +28,14 @@
 #ifndef BYTEORDER_H
 #define BYTEORDER_H
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
-#if defined (__alpha__) || defined (__i386__)
-	#define __LITTLEENDIAN__
-#elif defined (__mips__) || defined (__ppc__) || defined (__sparc__) \
-	|| defined (__hppa__)
+#if WORDS_BIGENDIAN
 	#define __BIGENDIAN__
+	#define _AF_BYTEORDER_NATIVE (AF_BYTEORDER_BIGENDIAN)
+#else
+	#define __LITTLEENDIAN__
+	#define _AF_BYTEORDER_NATIVE (AF_BYTEORDER_LITTLEENDIAN)
 #endif
 
 #ifndef uint16
