@@ -1,28 +1,28 @@
 /*
 	Audio File Library
-
-	Copyright 1998, Michael Pruett <michael@68k.org>
+	Copyright (C) 1998-1999, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
-	modify it under the terms of the GNU General Public License as
-	published by the Free Software Foundation; either version 2 of
-	the License, or (at your option) any later version.
+	modify it under the terms of the GNU Library General Public
+	License as published by the Free Software Foundation; either
+	version 2 of the License, or (at your option) any later version.
 
-	This library is distributed in the hope that it will be
-	useful, but WITHOUT ANY WARRANTY; without even the implied
-	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-	PURPOSE.  See the GNU General Public License for more details.
+	This library is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+	Library General Public License for more details.
 
-	You should have received a copy of the GNU General Public
-	License along with this program; if not, write to the Free
-	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-	MA 02111-1307, USA.
+	You should have received a copy of the GNU Library General Public
+	License along with this library; if not, write to the 
+	Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
+	Boston, MA  02111-1307  USA.
 */
 
 /*
 	loop.c
 
-	This file contains routines dealing with loops.
+	This file contains routines dealing with the Audio File Library's
+	internal loop data structures.
 */
 
 #include <stdlib.h>
@@ -31,7 +31,10 @@
 #include "audiofile.h"
 #include "afinternal.h"
 
-struct _Loop *initLoop (struct _Loop *loop)
+static struct _Loop *initLoop (struct _Loop *loop);
+static struct _Loop *findLoopByID (int id, struct _Loop *loops, int count);
+
+static struct _Loop *initLoop (struct _Loop *loop)
 {
 	assert(loop);
 
@@ -40,9 +43,11 @@ struct _Loop *initLoop (struct _Loop *loop)
 
 	loop->loopCount = 0;
 	loop->playMode = AF_LOOP_MODE_NOLOOP;
+
+	return loop;
 }
 
-struct _Loop *findLoopByID (int id, struct _Loop *loops, int count)
+static struct _Loop *findLoopByID (int id, struct _Loop *loops, int count)
 {
 	int	i;
 
@@ -113,7 +118,7 @@ void afSetLoopMode (AFfilehandle file, int instid, int loopid, int mode)
 	loop = findLoopByID(loopid, file->loops, file->loopCount);
 	if (loop == NULL)
 	{
-		/* error(AF_BAD_LOOPID); */
+		_af_error(AF_BAD_LOOPID);
 		return;
 	}
 
@@ -130,7 +135,7 @@ int afGetLoopMode (AFfilehandle file, int instid, int loopid)
 	loop = findLoopByID(loopid, file->loops, file->loopCount);
 	if (loop == NULL)
 	{
-		/* error(AF_BAD_LOOPID); */
+		_af_error(AF_BAD_LOOPID);
 		return -1;
 	}
 
@@ -147,16 +152,16 @@ int afSetLoopCount (AFfilehandle file, int instid, int loopid, int count)
 	loop = findLoopByID(loopid, file->loops, file->loopCount);
 	if (loop == NULL)
 	{
-		/* error(AF_BAD_LOOPID); */
+		_af_error(AF_BAD_LOOPID);
 		return -1;
 	}
 
 	loop->loopCount = count;
+	return 0;
 }
 
 int afGetLoopCount (AFfilehandle file, int instid, int loopid)
 {
-	int				i;
 	struct _Loop	*loop;
 
 	assert(file != NULL);
@@ -187,7 +192,6 @@ void afSetLoopStart (AFfilehandle file, int instid, int loopid, int markerid)
 
 int afGetLoopStart (AFfilehandle file, int instid, int loopid)
 {
-	int				i;
 	struct _Loop	*loop;
 
 	assert(file != NULL);
@@ -218,7 +222,6 @@ void afSetLoopEnd (AFfilehandle file, int instid, int loopid, int markerid)
 
 int afGetLoopEnd (AFfilehandle file, int instid, int loopid)
 {
-	int				i;
 	struct _Loop	*loop;
 
 	assert(file != NULL);
@@ -311,7 +314,7 @@ void afSetLoopTrack (AFfilehandle file, int instid, int loopid, int trackid)
 	loop = findLoopByID(loopid, file->loops, file->loopCount);
 	if (loop == NULL)
 	{
-		/* error(AF_BAD_LOOPID); */
+		_af_error(AF_BAD_LOOPID);
 	}
 }
 
@@ -326,7 +329,7 @@ int afGetLoopTrack (AFfilehandle file, int instid, int loopid)
 
 	if (loop == NULL)
 	{
-		/* error(AF_BAD_LOOPID); */
+		_af_error(AF_BAD_LOOPID);
 		return -1;
 	}
 
